<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class BLA_Plugin {

	public function __construct() {

		add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_tab' ), 100 );
		add_action( 'woocommerce_settings_bla', array( $this, 'output' ) );
		add_action( 'woocommerce_settings_save_bla', array( $this, 'save' ) );
		add_action( 'woocommerce_admin_field_bla_button', array( $this, 'field_bla_button' ) );

		add_action( 'add_meta_boxes', array( $this, 'register_meta_boxes' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_assets' ) );

		add_action( 'wp_ajax_bla_sync_product', array( $this, 'ajax_sync_product' ) );
		add_action( 'wp_ajax_bla_sync_products', array( $this, 'ajax_sync_products' ) );
		add_action( 'wp_ajax_bla_sync_order', array( $this, 'ajax_sync_order' ) );

		add_action( 'save_post', array( $this, 'maybe_sync_product' ), 99 );
		add_action( 'woocommerce_update_product_variation', array( $this, 'maybe_sync_variation' ), 99 );
		add_action( 'woocommerce_new_product_variation', array( $this, 'maybe_sync_variation' ), 99 );
		add_action( 'woocommerce_ajax_save_product_variations', array( $this, 'maybe_sync_variation' ), 99 );
		add_action( 'woocommerce_order_status_processing', array( $this, 'maybe_sync_order' ), 99 );
		add_action( 'woocommerce_order_status_completed', array( $this, 'maybe_sync_order' ), 99 );
		add_action( 'bla_send_order_to_bla', array( $this, 'sync_order' ), 99, 1 );
	}

    public function add_settings_tab( $settings_tabs )
    {
        $settings_tabs['bla'] = __( 'Björn Lundén Integration', 'woocommerce-bjorn-lunden-integration' );
        return $settings_tabs;
    }

	public function get_settings()
	{

		$current_section = '';

		$settings = array(

			array(
				'name' => __( 'Settings', 'woocommerce-bjorn-lunden-integration' ),
				'type' => 'title',
				'desc' => __( 'In order to connect your WooCommerce store to BLA you will need a public key.<br /><br /><h4>How to retrieve your public key</h4><ol><li>Go to your BL Administration client and choose the company you want to integrate WooCommerce with.</li><li>Click on "Arkiv" -> "BL Integration"</li><li>Click on "Aktivera" on "WooCommerce integrationen"</li><li>Get your public key by clicking on the cogwheel in the top right corner.</li><li>Add the public key inside of the "Public Key" field below.</li></ol>', 'woocommerce-bjorn-lunden-integration' ),
				'id'   => 'bla_settings_title',
			),
			array(
				'type'     => 'text',
				'id'       => 'bla_public_key',
				'name'     => __( 'Public Key', 'woocommerce-bjorn-lunden-integration' ),
				'desc'     => '',
			),
			array(
				'type'     => 'select',
				'id'       => 'bla_sync_orders',
				'name'     => __( 'Synchronize orders', 'woocommerce-bjorn-lunden-integration' ),
				'options'  => array(
					'auto' => __( 'Auto', 'woocommerce-bjorn-lunden-integration' ),
					'manually' 	=> __( 'Manually', 'woocommerce-bjorn-lunden-integration' )
				),
				'class'    => 'wc-enhanced-select',
				'default'  => 'auto'
			),
			array(
				'type'     => 'select',
				'id'       => 'bla_sync_articles',
				'name'     => __( 'Synchronize articles', 'woocommerce-bjorn-lunden-integration' ),
				'options'  => array(
					'auto' => __( 'Auto', 'woocommerce-bjorn-lunden-integration' ),
					'manually' 	=> __( 'Manually', 'woocommerce-bjorn-lunden-integration' )
				),
				'class'    => 'wc-enhanced-select',
				'default'  => 'auto'
			),
			array(
				'type' => 'sectionend',
				'id'   => 'bla_settings_section_end_1'
			),
			array(
				'name' => __( 'VAT-codes', 'woocommerce-bjorn-lunden-integration' ),
				'type' => 'title',
				'desc' => __( 'Please set the correct VAT-codes for the taxgroups you have created.', 'woocommerce-bjorn-lunden-integration' ),
				'id'   => 'bla_tax_codes_title',
			)
		);

		foreach( wc_get_product_tax_class_options() as $tax_id => $tax_name ) :

			if( !$tax_id ) $tax_id = "standard";

			$settings[] = array(
				'type'     => 'select',
				'id'       => 'bla_tax_'.$tax_id,
				'name'     => $tax_name,
				'options'  => array(
					'25' => __( '25%', 'woocommerce-bjorn-lunden-integration' ),
					'12' 	=> __( '12%', 'woocommerce-bjorn-lunden-integration' ),
					'6' 	=> __( '6%', 'woocommerce-bjorn-lunden-integration' ),
					'0' 	=> __( '0%', 'woocommerce-bjorn-lunden-integration' ),
				),
				'class'    => 'wc-enhanced-select',
				'default'  => '25'
			);

		endforeach;


		if ( get_option( "bla_public_key" ) ) {
			$settings = array_merge( $settings, array(
				array(
					'type' => 'sectionend',
					'id'   => 'bla_settings_section_end_2'
				),
				array(
					'name' => __( 'Synchronize', 'woocommerce-bjorn-lunden-integration' ),
					'type' => 'title',
					'desc' => '',
					'id'   => 'bla_title_sync',
				),
				array(
					'type' => 'bla_button',
					'id'   => 'bla_button_sync_articles',
					'action' => 'bla_sync_articles',
					'name' => __( 'Synchronize articles', 'woocommerce-bjorn-lunden-integration' )
				),
				array(
					'type' => 'sectionend',
					'id'   => 'bla_settings_section_end_3'
				)
			) );
		}
		else {
			$settings = array_merge( $settings, array(
				array(
					'type' => 'sectionend',
					'id'   => 'bla_settings_section_end_2'
				),
				array(
					'name' => __( 'Synchronize', 'woocommerce-bjorn-lunden-integration' ),
					'type' => 'title',
					'desc' => __( 'To synchronize your articles please add your public key first.', 'woocommerce-bjorn-lunden-integration'),
					'id'   => 'bla_title_sync',
				),

				array(
					'type' => 'sectionend',
					'id'   => 'bla_settings_section_end_3'
				)
			) );
		}


		return apply_filters( 'woocommerce_get_settings_bla', $settings, $current_section );

	}

	public function output()
	{

	    $settings = $this->get_settings();
	    WC_Admin_Settings::output_fields( $settings );
	}

	public function save()
	{

	    $settings = $this->get_settings();
	    WC_Admin_Settings::save_fields( $settings );
	}

	public function field_bla_button( $value )
	{

		$products_to_sync = BLA_Helpers::get_products_to_sync();
		$total = count( $products_to_sync );

		echo sprintf(
			'<tr valign="top">
				<th>
					<input id="%s" type="button" class="button %s" value="%s" data-action="%s" data-products="%s">
				</th>
				<td>%s products are waiting for synchronization.</td>
			</tr>',
			$value["id"],
			$total == 0 ? "disabled" : "",
			esc_attr( $value["name"] ),
			esc_attr( $value["action"] ),
			esc_attr( json_encode( $products_to_sync ) ),
			esc_attr( $total )
		);
	}

	public function register_meta_boxes()
	{
		if ( get_option( "bla_public_key" ) ) {
			add_meta_box( 'order_bla', __( 'Björn Lundén Integration', 'woocommerce-bjorn-lunden-integration' ), array( $this, "meta_box_order" ), 'shop_order', 'side' );
			add_meta_box( 'product_bla', __( 'Björn Lundén Integration', 'woocommerce-bjorn-lunden-integration' ), array( $this, "meta_box_product" ), 'product', 'side' );
		}

	}

	public function meta_box_order( $post )
	{
		$last_synced = get_post_meta( $post->ID, "_bla_last_synced", true );
		include BLA_DIR."/templates/meta_box/order.php";
	}

	public function meta_box_product( $post )
	{
			$product = wc_get_product( $post->ID );
			if( $product->is_type( "simple" ) || $product->is_type( "variable" ) ) :
				$last_synced = get_post_meta( $post->ID, "_bla_last_synced", true );
				include BLA_DIR."/templates/meta_box/product.php";
			else :
				include BLA_DIR."/templates/meta_box/product_invalid.php";
			endif;

	}

	public function enqueue_admin_assets()
	{
		wp_enqueue_script( "bla", plugins_url( "assets/js/admin.js", BLA_FILE ) );
		wp_enqueue_style( "bla", plugins_url( "assets/css/admin.css", BLA_FILE ) );
	}

	public function sync_product( $product_id ){

		$product = wc_get_product( $product_id );

		$products = array();

		if( $product ) :

			switch( $product->get_type() ) :

				case "simple" :

					$products[] = BLA_Helpers::get_product_data( $product_id );

				break;

				case "variable" :

					foreach( $product->get_children() as $variation_id ) :

						$products[] = BLA_Helpers::get_product_data( $variation_id );

					endforeach;

				break;

			endswitch;

		endif;

		$api = new BLA_API( get_option( "bla_public_key" ) );

		$response = $api->sync_products( $products );

		$is_error = false;
		$error_message = "";

		$last_synced_time = current_time( "mysql" );

		foreach( $response["products"] as $product ) :

			if( $product["status"] == "OK" ) :

				update_post_meta( $product["wp_id"],"_bla_id", $product["bla_id"] );
				if( $product["parent_wp_id"] ) :
					update_post_meta( $product["parent_wp_id"], "_bla_last_synced", $last_synced_time );
				else :
					update_post_meta( $product["wp_id"], "_bla_last_synced", $last_synced_time );
				endif;

			elseif( !$is_error ) :
				$is_error = true;
				$error_message = $product["error"];
			endif;

		endforeach;


		$response = array(
			"status" => $is_error?"error":"OK",
			"message" => $is_error?$error_message:"Product successfully synced",
			"last_synced_time" => $last_synced_time,
			"api" => $response
		);

		if( !$is_error ) :
			update_post_meta( $product_id, "_bla_last_synced", $last_synced_time );
		endif;

		return $response;
	}

	public function ajax_sync_product()
	{
		$product_id = $_POST["ID"];
		$response = $this->sync_product( $product_id );
		wp_send_json( $response );
	}

	private function sync_products( $product_ids ){

		$products = array();

		foreach( $product_ids as $product_id ) :

			$products[] = BLA_Helpers::get_product_data( $product_id );

		endforeach;

		$api = new BLA_API( get_option( "bla_public_key" ) );

		$response = $api->sync_products( $products );

		$is_error = false;
		$error_message = "";

		$last_synced_time = current_time( "mysql" );

		foreach( $response["products"] as $product ) :

			if( $product["status"] == "OK" ) :

				update_post_meta( $product["wp_id"],"_bla_id", $product["bla_id"] );
				if( $product["parent_wp_id"] ) :
					update_post_meta( $product["parent_wp_id"], "_bla_last_synced", $last_synced_time );
				else :
					update_post_meta( $product["wp_id"], "_bla_last_synced", $last_synced_time );
				endif;

			elseif( !$is_error ) :
				$is_error = true;
				$error_message = $product["error"];
			endif;

		endforeach;

		$response = array(
			"status" => $is_error?"error":"OK",
			"message" => $is_error?$error_message:"Product successfully synced",
			"last_synced_time" => $last_synced_time,
			"api" => $response
		);

		return $response;
	}

	public function ajax_sync_products()
	{
		$product_ids = $_POST["products"];
		$response = $this->sync_products( $product_ids );
		wp_send_json( $response );
	}

	public function sync_order( $order_id ){

		$api = new BLA_API( get_option( "bla_public_key" ) );

		$order = BLA_Helpers::get_order_data( $order_id );

		$response = $api->sync_order( $order );

		$is_error = false;
		$error_message = "";

		$last_synced_time = current_time( "mysql" );

		if( $response["status"] === "OK" ) :
			update_post_meta( $order_id, "_bla_id", $response["bla_id"] );
			update_post_meta( $order_id, "_bla_last_synced", $last_synced_time );
		endif;

		return array(
			"status" => $response["status"],
			"message" => $response["status"]=="OK"?"Order successfully synced":$response["error"],
			"last_synced_time" => $last_synced_time,
			"api" => $response
		);
	}

	public function ajax_sync_order()
	{
		$order_id = $_POST["ID"];
		$response = $this->sync_order( $order_id );
		wp_send_json( $response );
	}

	public function maybe_sync_product( $post_id ){

		if( !current_user_can( "edit_post", $post_id ) ) return;

		if( defined( "DOING_AUTOSAVE" ) && DOING_AUTOSAVE ) return;

		if( get_post_type( $post_id ) === "product" ) :

			update_post_meta( $post_id, "_bla_modified_date", date("Y-m-d H:i:s"), true );

			if( get_option( "bla_sync_articles" ) === "auto" ) :

				$response = $this->sync_product( $post_id );

			endif;

		endif;
	}

	public function maybe_sync_variation( $variation_id ){

		if( get_option( "bla_sync_articles" ) === "auto" ) :

			$response = $this->sync_product( $variation_id );

		endif;
	}

	public function maybe_sync_order( $order_id ){

   		if ( ! $order_id )
        	return;

		if( get_option( "bla_sync_orders" ) === "auto" && !get_post_meta( $order_id, "_bla_id", true ) ) :

			$response = $this->sync_order( $order_id );

		endif;
	}

}
